#include "init.h"
#include "stm8s_gpio.h"
#include "stm8s.h"
#include "timebase.h"
#include "motor.h"
#include "control.h"
#include <stdbool.h>

Motor_Status_Type motor_status = MOTOR_STOPPED;// 
Drive_Command_Type curr_cmd = CMD_NONE;//  
uint8_t relay_lock = 0;
bool power_on = false; //  
bool relay_on = false;//  
bool curr_auto_stop = false;//   

Motor_Pos_Type cur_motor_pos = POS_UNKNOWN;

bool motor_fatal_error = false;// ,  


extern bool open_sensor;
extern bool close_sensor;

extern bool ac_dc_on;//  

//   
Drive_Command_Type save_cmd;
uint16_t save_timeout;

void set_motor_speed(uint8_t value)
{
  TIM2_SetCounter(0);
  TIM2_SetCompare1((uint16_t)value);
}

void set_relay_open(void)
{
  GPIO_WriteLow(RELAY_PORT, RELAY_PIN);
  relay_on = false;
}

void set_relay_close(void)
{
  GPIO_WriteHigh(RELAY_PORT, RELAY_PIN);
  relay_on = true;
}

void enable_power(void)
{
  GPIO_WriteHigh(STAND_BY_PORT, STAND_BY_PIN);
  power_on = true;
}

//      
void motor_beep(void)
{
  set_motor_speed(60);// 
  set_motor_timer(200);
  curr_auto_stop = false;
  motor_status = MOTOR_RUNING;
}

void disable_power(void)
{
  GPIO_WriteLow(STAND_BY_PORT, STAND_BY_PIN);
  power_on = false;
}

void stop_motor(void)
{
  set_motor_speed(0);
  motor_status = MOTOR_STOPPED;
  curr_cmd = CMD_STOP;
  curr_auto_stop = false;
  set_motor_timer(100);//  
  //  
}

//   
bool auto_start_drive(Drive_Command_Type cmd)
{
  if (motor_status != MOTOR_STOPPED) {return false;} // 
  if ((cur_motor_pos == POS_OPENED) && (cmd == CMD_OPEN)) {return false;} // 
  if ((cur_motor_pos == POS_CLOSED) && (cmd == CMD_CLOSE)) {return false;} // 
  start_drive(cmd, 20000,true);
  return true;
}

// 
//cmd - 
//timeout -   
//auto_stop -   
void start_drive(Drive_Command_Type cmd, uint16_t timeout,bool auto_stop)
{
  if (motor_fatal_error == true) {return;}//     
  if (ac_dc_on == false) {return;}//     
  
  if (motor_status == MOTOR_WAITING_POWER) {return;}//,      
  if (motor_status == MOTOR_WAITING_RELAY) {return;}
    
  curr_auto_stop = auto_stop;
  
  if (power_on == false) //  
  {
    enable_power();
    motor_status = MOTOR_WAITING_POWER;
    set_power_timer(1000);//   
    save_cmd = cmd;
    save_timeout = timeout;
    return;
  }
  
  if ((auto_stop == true))
  {
    // ,    
    if ((open_sensor == true) && (cmd == CMD_OPEN)) {motor_beep();return;}
    if ((close_sensor == true) && (cmd == CMD_CLOSE)) {motor_beep();return;}
  }
  
  bool old_relay_status = relay_on;
  if (cmd == CMD_OPEN){set_relay_open();} else {set_relay_close();}
  if (old_relay_status != relay_on) //  ,    
  {
    // 
    motor_status = MOTOR_WAITING_RELAY;
    set_motor_timer(100);//   
    save_cmd = cmd;
    save_timeout = timeout;
    return;//  
  }
  
  curr_cmd = cmd;
  set_button_timer(50);// 
  set_motor_speed(201);// 
  set_motor_timer(timeout);
  motor_status = MOTOR_RUNING;
}


void motor_handler(void)
{
  if (motor_timer_elapsed() && (motor_status == MOTOR_RUNING))//   
  {
    if ((curr_auto_stop == true) && (motor_fatal_error == false)) {motor_fatal_error = true;}
    stop_motor();    
  }
  
  if (motor_timer_elapsed() && (motor_status == MOTOR_WAITING_RELAY))//  
  {
    motor_status = MOTOR_STOPPED;
    start_drive(save_cmd, save_timeout,curr_auto_stop);
  }
  
  if (power_timer_elapsed() && (motor_status == MOTOR_WAITING_POWER))//  
  {
    motor_status = MOTOR_STOPPED;
    start_drive(save_cmd, save_timeout,curr_auto_stop);
    set_power_timer(12000);//  ,     
  }
  
  if (motor_status == MOTOR_RUNING)
  {
    set_power_timer(12000);//  (+32),     
    if (relay_on == true) {set_relay_timer(6000);}//   6 
  }
  
  if (power_timer_elapsed() && (motor_status == MOTOR_STOPPED) &&(power_on == true))//  
  {
    disable_power();
  }
  
  if ((relay_on == true) && relay_timer_elapsed() && (motor_status == MOTOR_STOPPED)) 
  {
    GPIO_WriteLow(RELAY_PORT, RELAY_PIN);
    relay_on = false;
  }
  
  //    
  if (open_sensor == true) {cur_motor_pos = POS_OPENED;}
  else 
  if (close_sensor == true) {cur_motor_pos = POS_CLOSED;}
  else {cur_motor_pos = POS_UNKNOWN;}
    
  
  if ((curr_auto_stop == true) && (motor_status == MOTOR_RUNING) && button_timer_elapsed())// 
  {
    if ((cur_motor_pos == POS_OPENED) && (curr_cmd == CMD_OPEN)) {
      stop_motor();}
    if ((cur_motor_pos == POS_CLOSED) && (curr_cmd == CMD_CLOSE)) {
      stop_motor();}
  }
}

